import ctypes
from ctypes import *
from pydbus import SystemBus
from gi.repository import GLib, Gtk, Gio
import gi
from enum import Enum

class SoftwareInstallControlMode(Enum):
    CLOSE_MODE = 0
    WHITELIST_MODE = 1
    BLACKLIST_MODE = 2

class SoftwareInstallControlType(Enum):
    WHITELIST = 1
    BLACKLIST = 2

class SoftwareUninstallControlMode(Enum):
    CLOSE_MODE = 0
    OPEN_MODE = 1

class SoftwareInstallControl:
    """
    <node>
        <interface name='com.kylin.softwarectrl.install'>
            <method name='set_install_mode' >
                <arg type='i' direction='in' name='mode'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='get_install_mode' >
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='add_install_black_package' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='add_install_white_package' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='del_install_black_package' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
             <method name='del_install_white_package' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='add_install_black_package_list' >
                <arg type='as' direction='in' name='package_list'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='add_install_white_package_list' >
                <arg type='as' direction='in' name='package_list'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='clear_install_package_list' >
                <arg type='i' direction='out' name='ret'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        self.bwlist_object = CDLL("libkylin_bwlist.so")
        self._black = create_string_buffer(b"black")
        self._white = create_string_buffer(b"white")

    def set_install_mode(self, mode):
        if self.bwlist_object.kylin_bwlist_create() == -1:
            return -1;
        else:
            if mode == SoftwareInstallControlMode.CLOSE_MODE.value:
                return self.bwlist_object.kylin_bwlist_set_root_distinguish(0)
            elif mode == SoftwareInstallControlMode.WHITELIST_MODE.value:
                return self.bwlist_object.kylin_bwlist_set_root_distinguish(1)
            elif mode == SoftwareInstallControlMode.BLACKLIST_MODE.value:
                return self.bwlist_object.kylin_bwlist_set_root_distinguish(2)
            else:
                return -1;

    def get_install_mode(self):
        return self.bwlist_object.kylin_bwlist_get_root_distinguish()

    def add_install_black_package(self, package):
        package = create_string_buffer(package.encode())
        return self.bwlist_object.kylin_bwlist_add(self._black, package)
    
    def add_install_white_package(self, package):
        package = create_string_buffer(package.encode())
        return self.bwlist_object.kylin_bwlist_add(self._white, package)

    def del_install_black_package(self, package):
        package = create_string_buffer(package.encode())
        return self.bwlist_object.kylin_bwlist_del(self._black, package)

    def del_install_white_package(self, package):
        package = create_string_buffer(package.encode())
        return self.bwlist_object.kylin_bwlist_del(self._white, package)

    def add_install_black_package_list(self, package_list):
        for package in package_list:
            package = create_string_buffer(package.encode())
            ret = self.bwlist_object.kylin_bwlist_add(self._black, package)
            if ret == -1:
                return -1;
        return 0;
    
    def add_install_white_package_list(self, package_list):
        for package in package_list:
            package = create_string_buffer(package.encode())
            ret = self.bwlist_object.kylin_bwlist_add(self._white, package)
            if ret == -1:
                return -1;
        return 0;
    
    def clear_install_package_list(self):
        return self.bwlist_object.kylin_bwlist_clear()



class SoftwareUninstallControl:
    """
    <node>
        <interface name='com.kylin.softwarectrl.uninstall'>
            <method name='set_uninstall_mode' >
                <arg type='i' direction='in' name='mode'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='get_uninstall_mode' >
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='add_uninstall_black_package' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='del_uninstall_black_package' >
                <arg type='s' direction='in' name='package'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='add_uninstall_black_package_list' >
                <arg type='as' direction='in' name='package_list'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='clear_uninstall_package_list' >
                <arg type='i' direction='out' name='ret'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        self.udeblist_object = CDLL("libkylin_bwlist.so")

    def set_uninstall_mode(self, mode):
        if self.udeblist_object.kylin_udeblist_create() == -1:
            return -1;
        else:
            if mode == SoftwareUninstallControlMode.CLOSE_MODE.value:
                return self.udeblist_object.kylin_udeblist_set_root_distinguish(0)
            elif mode == SoftwareUninstallControlMode.OPEN_MODE.value:
                return self.udeblist_object.kylin_udeblist_set_root_distinguish(1)
            else:
                return -1;

    def get_uninstall_mode(self):
        return self.udeblist_object.kylin_udeblist_get_root_distinguish()

    def add_uninstall_black_package(self, package):
        package = create_string_buffer(package.encode())
        return self.udeblist_object.kylin_udeblist_add(package)

    def del_uninstall_black_package(self, package):
        package = create_string_buffer(package.encode())
        return self.udeblist_object.kylin_udeblist_del(package)

    def add_uninstall_black_package_list(self, package_list):
        for package in package_list:
            package = create_string_buffer(package.encode())
            ret = self.udeblist_object.kylin_udeblist_add(package)
            if ret == -1:
                return -1;
        return 0;

    def clear_uninstall_package_list(self):
        return self.udeblist_object.kylin_udeblist_clear()


if __name__ == '__main__':
    loop = GLib.MainLoop()
    bus = SystemBus()
    bus.publish("com.kylin.softwarectrl", SoftwareInstallControl(), SoftwareUninstallControl())
    loop.run()